@@ -1,9 +1,11 @@
 //! Bindings to the client library `libwayland-client.so`
 //!
-//! The generated handle is named `WAYLAND_CLIENT_HANDLE`
+//! The generated handle is named `wayland_client_handle()`
 
 #![cfg_attr(rustfmt, rustfmt_skip)]
 
+#[cfg(all(feature = "client", feature = "dlopen"))]
+use once_cell::sync::Lazy;
 #[cfg(feature = "client")]
 use super::common::*;
 #[cfg(feature = "client")]
@@ -45,7 +47,7 @@ external_library!(WaylandClient, "wayland-client",
     // proxys
         fn wl_proxy_create(*mut wl_proxy, *const wl_interface) -> *mut wl_proxy,
         fn wl_proxy_destroy(*mut wl_proxy) -> (),
-        fn wl_proxy_add_listener(*mut wl_proxy, *mut extern fn(), *mut c_void) -> c_int,
+        fn wl_proxy_add_listener(*mut wl_proxy, *mut extern "C" fn(), *mut c_void) -> c_int,
         fn wl_proxy_get_listener(*mut wl_proxy) -> *const c_void,
         fn wl_proxy_add_dispatcher(*mut wl_proxy, wl_dispatcher_func_t, *const c_void, *mut c_void) -> c_int,
         fn wl_proxy_marshal_array_constructor(*mut wl_proxy, u32, *mut wl_argument, *const wl_interface) -> *mut wl_proxy,
@@ -84,16 +86,9 @@ external_library!(WaylandClient, "wayland-client",
 );
 
 #[cfg(all(feature = "client", feature = "dlopen"))]
-lazy_static::lazy_static!(
-    pub static ref WAYLAND_CLIENT_OPTION: Option<WaylandClient> = {
-        // This is a workaround for Ubuntu 17.04, which doesn't have a bare symlink
-        // for libwayland-client.so but does have it with the version numbers for
-        // whatever reason.
-        //
-        // We could do some trickery with str slices but that is more trouble
-        // than its worth
-        let versions = ["libwayland-client.so",
-                        "libwayland-client.so.0"];
+pub fn wayland_client_option() -> Option<&'static WaylandClient> {
+    static WAYLAND_CLIENT_OPTION: Lazy<Option<WaylandClient>> = Lazy::new(||{
+        let versions = ["libwayland-client.so.0", "libwayland-client.so"];
         for ver in &versions {
             match unsafe { WaylandClient::open(ver) } {
                 Ok(h) => return Some(h),
@@ -105,11 +100,17 @@ lazy_static::lazy_static!(
             }
         }
         None
-    };
-    pub static ref WAYLAND_CLIENT_HANDLE: &'static WaylandClient = {
-        WAYLAND_CLIENT_OPTION.as_ref().expect("Library libwayland-client.so could not be loaded.")
-    };
-);
+    });
+
+    WAYLAND_CLIENT_OPTION.as_ref()
+}
+
+#[cfg(all(feature = "client", feature = "dlopen"))]
+pub fn wayland_client_handle() -> &'static WaylandClient {
+    static WAYLAND_CLIENT_HANDLE: Lazy<&'static WaylandClient> = Lazy::new(|| wayland_client_option().expect("Library libwayland-client.so could not be loaded."));
+
+    &WAYLAND_CLIENT_HANDLE
+}
 
 #[cfg(all(feature = "client", not(feature = "dlopen")))]
 pub fn is_lib_available() -> bool {
@@ -117,5 +118,5 @@ pub fn is_lib_available() -> bool {
 }
 #[cfg(all(feature = "client", feature = "dlopen"))]
 pub fn is_lib_available() -> bool {
-    WAYLAND_CLIENT_OPTION.is_some()
+    wayland_client_option().is_some()
 }
